@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/display.c,v 1.63 2006/05/01 17:26:26 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/display.c,v 1.64 2006/08/07 23:53:21 richb Exp $
  *
  *  Copyright (c) 1987-2006 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -152,6 +152,7 @@ initialise()
     /* TODO: perhaps this function should be renamed to reset. */
 
     int i;
+    struct exprm_state *e;
  
     v->error         = 0;           /* Currently no display error. */
     v->cur_op        = '?';         /* No arithmetic operator defined yet. */
@@ -164,7 +165,7 @@ initialise()
   
     v->new_input = 1;               /* Value zero is on calculator display */
 
-	struct exprm_state *e = get_state();
+    e = get_state();
     free(e->expression);
     e->expression = NULL;
 }
