@@ -1,5 +1,5 @@
 
-/*  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
+/*  Copyright (c) 1987-2006 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -820,3 +820,32 @@ calc_rshift(int s[MP_SIZE], int t[MP_SIZE], int times, enum shiftd dir)
     dval = setbool(temp);
     mpcdm(&dval, t);
 }
+
+
+int
+is_integer(int MPnum[MP_SIZE])
+{    
+    int MP1[MP_SIZE];
+    mpcmim(MPnum, MP1);
+    return mpeq(MPnum, MP1);
+}
+
+int
+is_natural(int MPnum[MP_SIZE])
+{    
+    int MP1[MP_SIZE];
+    if (!is_integer(MPnum)) {
+	return 0;
+    }
+    mpabs(MPnum, MP1);
+    return mpeq(MPnum, MP1);
+}
+
+void
+calc_epowy(int s[MP_SIZE], int t[MP_SIZE])
+{
+    int MP1[MP_SIZE];
+    
+    mpstr(s, MP1);
+    mpexp(MP1, t);
+}
